/**
 * Written by Dvaid Smith
 * 
 * This program scans the directories given on the command line for any pulsar observations
 * For every observation it finds, it checks that it exists in the database
 * If an observation exists on disk but not in the database, it is added
 * If an observation is in the database but not on disk, then we delete it
 * 
 * An observation can be in many forms
 * 1) A single PSRFITS archive ending in rf or cf
 * 2) A set of files in a CPSR2 directory ending with cfb and beginning with either m, n or o ( 1 file for each sub integration)
 * 3) If an observation is in a directory that ends with _R then it is a calibration
 * 4) If the fits file has a pulsar src_name of CAL then it is also a calibration file (check with dick about this)
 **/


#include <iostream>
#include <vector>
#include <string>
#include <dirutil.h>
#include <strutil.h>
#include <boost/regex.hpp>
#include "psrdb.h"



using std::cerr;
using std::cout;
using std::endl;
using std::vector;
using std::string;
using mysqlpp::Connection;
using mysqlpp::Query;
using mysqlpp::Exception;


const char *database = "psrchive";
const char *db_username = "psrdba";
const char *db_password = "lighthouse";
const char *db_hostname = "localhost";

psrdb db;




/**
 * Take the files given to be a calibration and add an entry to the database
 * if none exists yet, otherwise mark the calibartion entry as present.
 **/

void AddCalEntry( const vector<string> &files )
{
  /**
   * if we have a single file
   *   search the cals table for that file
   *   if we have a match
   *     mark it as present
   *   otherwise
   *     create a new entry in the database for that file
   * otherwise
   *   determine the band prefix from the first filename
   *   search the cals table for a matching directory/band prefix
   *   if we have a match
   *     mark it as present
   *   otherwise
   *     create a new entry ni the database for that file
   **/
  
  cerr << "Adding a cal entry for " << files.size() << " files under " << pathname(files.front()) << endl;
}

void AddCalEntry( const string &single_file )
{
  vector<string> c;
  c.push_back( single_file );
  AddCalEntry( c );
}



/**
 * Take the files given to be an observation and add an entry in the database
 * if none exists yet, otherwise mark the observation entry as present
 **/

void AddObservationEntry( const vector<string> &files )
{
  cerr << "Adding a observation entry for " << files.size() << " files under " << files.front() << endl;
  
  ObservationEntry entry;
  
  string example_datafile = basename( files.front() );
  
  string full_path = pathname( files.front() );
  string stripped_path = full_path.substr( 0, full_path.size() - 1 );
  
  string new_data_loc = pathname( stripped_path );
  string new_filename = basename( stripped_path );
  
  entry.filename = new_filename;
  entry.data_loc = new_data_loc;
  entry.prefix = example_datafile[0];
  entry.files = files;
  
  db.CheckObservation( entry );
}

void AddObservationEntry( const string &single_file )
{
  ObservationEntry entry;
  
  entry.filename = basename( single_file );
  entry.data_loc = pathname( single_file );
  entry.prefix = '\0';

  db.CheckObservation( entry );
}



/**
 * This function iteratively examines a directory and calls itself to examine
 * any subdirectories
 **/

void ExamineDirectory( const string path )
{
  vector<string> rf_files;
  vector<string> cf_files;
  vector<string> m_fb_files;
  vector<string> n_fb_files;
  vector<string> m_cfb_files;
  vector<string> n_cfb_files;
  vector<string> r_cfb_files;
  
  boost::regex const rf_matcher(".*\\.rf");
  boost::regex const cf_matcher(".*\\.cf");
  boost::regex const m_fb_matcher("m.*\\.fb");
  boost::regex const n_fb_matcher("n.*\\.fb");
  boost::regex const m_cfb_matcher("m.*\\.cfb");
  boost::regex const n_cfb_matcher("n.*\\.cfb");
  boost::regex const r_cfb_matcher("r.*\\.cfb");

  vector<string> contents;
  dirglob( &contents, path + string( "/*" ) );
  vector<string>::iterator it;
  for( it = contents.begin(); it != contents.end(); it ++ )
  {
    string filename = basename( (*it) );
    if( file_is_directory( (*it).c_str() ) )
    {
      if( filename.find("_S") != string::npos )
        cerr << "ignoring scrunch directory " << (*it) << endl;
      else
        ExamineDirectory( (*it) );
    }
    else
    {
      if( boost::regex_match( filename, rf_matcher ) )
        rf_files.push_back( (*it) );
      else if( boost::regex_match( filename, cf_matcher ) )
        cf_files.push_back( (*it) );
      else if( boost::regex_match( filename, m_fb_matcher ) )
        m_fb_files.push_back( (*it) );
      else if( boost::regex_match( filename, n_fb_matcher ) )
        n_fb_files.push_back( (*it) );
      else if( boost::regex_match( filename, m_cfb_matcher ) )
        m_cfb_files.push_back( (*it) );
      else if( boost::regex_match( filename, n_cfb_matcher ) )
        n_cfb_files.push_back( (*it) );
      else if( boost::regex_match( filename, r_cfb_matcher ) )
        r_cfb_files.push_back( (*it) );
//       else
//         cerr << "don't know how to handle file " << filename << endl;
    }
  }
//   cerr << "Found " << rf_files.size() << " rf files " << endl;
//   cerr << "Found " << cf_files.size() << " cf files " << endl;
//   cerr << "Found " << m_fb_files.size() << " m*.fb files " << endl;
//   cerr << "Found " << n_fb_files.size() << " n*.fb files " << endl;
//   cerr << "Found " << m_cfb_files.size() << " m*.cfb files " << endl;
//   cerr << "Found " << n_cfb_files.size() << " n*.cfb files " << endl;
//   cerr << "Found " << r_cfb_files.size() << " r*.cfb files " << endl;
  
  /**
   *   for each cf file
   *     add this file as a calibration
   *   for each rf file
   *     if this directory ends in _R
   *       add this file as a calibration
   *     otherwise
   *       add this file as an observation
   * for each of the vectors n*.cfb m*.cfb r*.cfb m*.fb n*.fb
   *   if we have any m*.cfb files
   *     if this directory ends in _R
   *       add these files as a calibration
   *     otherwise
   *       add these files as an observation
   **/
  
  for( it = cf_files.begin(); it != cf_files.end(); it ++ )
    AddCalEntry( (*it) );
  
  for( it = rf_files.begin(); it != rf_files.end(); it ++ )
    AddObservationEntry( (*it) );
  
  vector< vector<string> > cpsr2_observations;
  cpsr2_observations.push_back( m_cfb_files );
  cpsr2_observations.push_back( n_cfb_files );
  cpsr2_observations.push_back( r_cfb_files );
  cpsr2_observations.push_back( m_fb_files );
  cpsr2_observations.push_back( n_fb_files );
  
  boost::regex const r_matcher( "/.*_R\\//" );
  
  vector< vector<string> >::iterator oit;
  for( oit = cpsr2_observations.begin(); oit != cpsr2_observations.end(); oit ++ )
  {
    if( (*oit).size() != 0 )
    {
      if( boost::regex_match( (*oit).front(), r_matcher ) )
      {
        AddCalEntry( (*oit) );
      }
      else
      {
        AddObservationEntry( (*oit) );
      }
    }
  }
  
}



int main( int argc, char *argv[] )
{
  try
  {
    db.Connect( database, db_hostname, db_username, db_password );
    
    db.MarkObservations();
    
    vector<string> target_directories;
    for( int i = 1; i < argc; i ++ )
      ExamineDirectory( string( argv[i] ) );

    db.PurgeMissingObservations();
    db.PurgeMissingCals();
  }
  catch( Exception e )
  {
    cerr << "The following exception was caught: ";
    cerr << e.what() << endl;
  }
        
  return 0;
}

