


#include "psrdb.h"
#include <mysql++.h>
#include <Pulsar/Archive.h>


using mysqlpp::Query;
using mysqlpp::Result;
using mysqlpp::Row;
using std::cerr;
using std::endl;
using Pulsar::Archive;





/// Constructor

psrdb::psrdb()
{
}


/**
 * Connect to the pulsararchive database using the parameters given,
 * If we failed to connect for any reason, simply throw an exception.
 **/

void psrdb::Connect( const char *db_name,
                     const char *db_hostname,
                     const char *db_username,
                     const char *db_password )
{
  if( !db.connect( db_name, db_hostname, db_username, db_password ) )
    throw( FailedConnection() );
}



/**
 * Mark all the entries in the observations and cals tables as being
 * asbent. As we update entries we will mark the as present, and we
 * will know any that don't get marked as present can be deleted from
 * the database.
 **/

void psrdb::MarkObservations( void )
{
  Query obs_query = db.query();
  
  obs_query << "UPDATE observations SET present = 0 WHERE deleted = 0";
  obs_query.execute();
  
  Query cals_query = db.query();
  cals_query << "UPDATE cals SET present = 0 WHERE deleted = 0";
  cals_query.execute();
}



/**
 * Purge any records in the observations table that haven't been marked as present.
 **/

void psrdb::PurgeMissingObservations()
{
  Query purge_query = db.query();
  
  purge_query << "UPDATE observations SET deleted = 1 WHERE present = 0 AND deleted = 0";
  
  purge_query.execute();
}


/**
 * Purge any records in the cals table that haven't been marked as present
 **/

void psrdb::PurgeMissingCals()
{
  Query purge_query = db.query();
  
  purge_query << "UPDATE cals SET deleted = 1 WHERE present = 0 AND deleted = 0";
  
  purge_query.execute();
}


void psrdb::GetArchiveDetails( ObservationDetails &details, Reference::To<Archive> arch )
{
  int nbin = arch->get_nbin();
  int npol = arch->get_npol();
  int nchan = arch->get_nchan();
  
  details.obsfreq = arch->get_centre_frequency();
  details.bw = arch->get_bandwidth();
}

/**
 * This function retreives the details of an observation given the data_loc/filename/prefix
 **/

void psrdb::GetObsDetails( const ObservationEntry &entry, ObservationDetails &details )
{
  if( entry.prefix == '\0' )
  {
    string archive_path = entry.data_loc + entry.filename;
    Reference::To<Archive> arch = Archive::load( archive_path );
    if( !arch )
      throw( GenericException( (string("Failed to load archive") + archive_path).c_str() ) );
    
    GetArchiveDetails( details, arch );
  }
  else
  {
    Reference::To<Archive> first_arch = Archive::load( entry.files.front() );
    if( !first_arch )
      throw( GenericException( (string("Failed to load archive") + entry.files.front()).c_str() ) );

    GetArchiveDetails( details, first_arch );
  }
}



/**
 * This function checks that an observation already exists in the database, if
 * not it will create an entry for it.
 **/

void psrdb::CheckObservation( const ObservationEntry &entry )
{
  Query q = db.query();
  
  cerr << "checking that " << entry.filename << " exists" << endl;
  
  q << " SELECT file_id ";
  q << " FROM observations ";
  q << " WHERE data_loc = \"" << entry.data_loc << "\"";
  q << " AND filename = \"" << entry.filename << "\"";
  q << " AND prefix = \"" << entry.prefix << "\"";
  q << " AND deleted = 0 ";
  
  Result res = q.store();
  
  if( res.num_rows() != 0 )
  {
    Row row = res.fetch_row();
    string file_id = row["file_id"].get_string();

    Query present_query = db.query();
    present_query << " UPDATE observations SET present = 1 WHERE file_id = " << file_id;
    present_query.execute();
  }
  else
  {
    ObservationDetails details;
    GetObsDetails( entry, details );
    
    Query insert_query = db.query();
    insert_query << " INSERT into observations(data_loc,filename,prefix,obsfreq,bw,present) ";
    insert_query << " VALUES( \"" << entry.data_loc << "\", ";
    insert_query << " \"" << entry.filename << "\", ";
    insert_query << " '" << entry.prefix << "', ";
    insert_query << details.obsfreq << ", ";
    insert_query << details.bw << ", ";
    insert_query << " 1 )"; // present = 1
    
    insert_query.execute();
    
    cerr << entry.data_loc << "/" << entry.filename << " not in the database " << endl;
  }
}

